"""
Draft Sport Python
Player Points Module
author: hugh@blinkybeach.com
"""
from nozomi import Decodable, Immutable
from draft_sport.fantasy.scores.player.round import Round
from typing import TypeVar, Type, Any, List

T = TypeVar('T', bound='Points')


class Points(Decodable):

    def __init__(
        self,
        average_points: int,
        total_points: int,
        points_last_round: int,
        points_per_minute_played: int,
        rounds: List[Round]
    ) -> None:

        self._average_points = average_points
        self._total_points = total_points
        self._points_last_round = points_last_round
        self._points_per_minute_played = points_per_minute_played
        self._rounds = rounds

        return

    average_points = Immutable(lambda s: s._average_points)
    total_points = Immutable(lambda s: s._total_points)
    points_last_round = Immutable(lambda s: s._points_last_round)
    points_per_minute_played = Immutable(lambda s: s._points_per_minute_played)
    rounds = Immutable(lambda s: s._rounds)

    @classmethod
    def decode(cls: Type[T], data: Any) -> T:
        return cls(
            points_per_minute_played=data['points_per_minute_played'],
            average_points=data['average_points'],
            total_points=data['total_points'],
            points_last_round=data['points_last_round'],
            rounds=Round.decode_many(data['rounds'])
        )



"""
Draft Sport Python
Session Module
author: hugh@blinkybeach.com
"""
from nozomi.app import Session as NozomiAppSession
from nozomi import NozomiTime, Immutable, Configuration, Agent
from nozomi import StandaloneAgent, RequestCredentials
from typing import Optional, Type, TypeVar, Any
from draft_sport.security.perspective import Perspective
from nozomi import URLParameter, URLParameters, HTTPMethod, ApiRequest

T = TypeVar('T', bound='Session')


class Session(NozomiAppSession):

    API_PATH = '/session'

    def __init__(
        self,
        session_id: str,
        session_key: str,
        api_key: str,
        agent: Agent,
        created: NozomiTime,
        last_utilised: NozomiTime,
        perspective: Perspective
    ) -> None:

        assert isinstance(session_id, str)
        assert isinstance(session_key, str)
        assert isinstance(api_key, str)
        assert isinstance(agent, Agent)
        assert isinstance(created, NozomiTime)
        assert isinstance(last_utilised, NozomiTime)
        assert isinstance(perspective, Perspective)

        self._session_id = session_id
        self._session_key = session_key
        self._api_key = api_key
        self._agent = agent
        self._created = created
        self._last_utilised = last_utilised
        self._perspective = perspective

        return

    agent: Agent = Immutable(lambda s: s._agent)
    perspective: Perspective = Immutable(lambda s: s._perspective)
    api_key: str = Immutable(lambda s: s._api_key)
    session_id: int = Immutable(lambda s: s._session_id)

    agent_id = Immutable(lambda s: s._agent.agent_id)

    def delete(
        self,
        credentials: RequestCredentials,
        configuration: Configuration
    ) -> None:
        """Delete this Session, AKA logout the user"""
        target = URLParameter('session_id', str(self._session_id))
        parameters = URLParameters([target])

        ApiRequest(
            path=self.API_PATH,
            method=HTTPMethod.DELETE,
            configuration=configuration,
            credentials=credentials,
            data=None,
            url_parameters=parameters
        )
        return None

    @classmethod
    def retrieve(
        cls: Type[T],
        session_id: str,
        credentials: RequestCredentials,
        configuration: Configuration
    ) -> Optional[T]:
        """Return a Session with the given Session ID, if it exists"""

        assert isinstance(session_id, str)

        target = URLParameter('session_id', str(session_id))
        parameters = URLParameters([target])

        request = ApiRequest(
            path=cls.API_PATH,
            method=HTTPMethod.GET,
            configuration=configuration,
            credentials=credentials,
            data=None,
            url_parameters=parameters
        )

        if request.response_data is None:
            return None

        return cls.decode(request.response_data)

    @classmethod
    def create(
        cls: Type[T],
        email: str,
        secret: str,
        configuration: Configuration
    ) -> T:

        data = {
            'email': email,
            'secret': secret
        }

        raise NotImplementedError

    @classmethod
    def decode(cls: Type[T], data: Any) -> T:
        """Return a Session decoded from API response data"""

        return cls(
            data['session_id'],
            data['session_key'],
            data['api_key'],
            StandaloneAgent.decode(data),
            NozomiTime.decode(data['created']),
            NozomiTime.decode(data['last_utilised']),
            Perspective.with_id(data['perspective'])
        )



"""
Draft Sport
League Module
author: hugh@blinkybeach.com
"""
from nozomi import Immutable, Configuration, RequestCredentials
from nozomi import Decodable, NozomiTime
from typing import Optional, Type, TypeVar, Any, List
from nozomi import URLParameter, URLParameters, HTTPMethod, ApiRequest
from draft_sport.leagues.team import Team

T = TypeVar('T', bound='League')


class League(Decodable):

    _PATH = '/league'

    def __init__(
        self,
        public_id: str,
        teams: List[Team],
        commissioner_id: str,
        name: str,
        created: NozomiTime
    ) -> None:

        self._public_id = public_id
        self._teams = teams
        self._commissioner_id = commissioner_id
        self._name = name
        self._created = created

        return

    name = Immutable(lambda s: s._name)
    public_id = Immutable(lambda s: s._public_id)

    @classmethod
    def retrieve(
        cls: Type[T],
        public_id: str,
        credentials: RequestCredentials,
        configuration: Configuration
    ) -> Optional[T]:
        """
        Optionally return a League with the given public ID, if it exists
        """

        assert isinstance(public_id, str)

        parameters = URLParameters([
            URLParameter('league', public_id),
            URLParameter('season', '2020')  # MVP hardcode
        ])

        request = ApiRequest(
            path=cls._PATH,
            method=HTTPMethod.GET,
            configuration=configuration,
            data=None,
            url_parameters=parameters,
            credentials=credentials
        )

        return cls.optionally_decode(request.response_data)

    @classmethod
    def decode(cls: Type[T], data: Any) -> T:
        return cls(
            public_id=data['public_id'],
            teams=Team.decode_many(data['teams']),
            commissioner_id=data['commissioner_id'],
            name=data['name'],
            created=NozomiTime.decode(data['created'])
        )



"""
Draft Sport Python
Player Order By Enumeration
author: hugh@blinkybeach.com
"""
from enum import Enum


class OrderBy(Enum):

    PLAYER_NAME = 'player_name',
    AVERAGE_POINTS = 'average_points'
    POINTS_LAST_ROUND = 'points_last_round'
    TOTAL_SEASON_POINTS = 'total_season_points'



"""
Draft Sport Python
Fantasy Player Score Module
Author: hugh@blinkybeach.com
"""
from nozomi import Decodable, HTTPMethod, URLParameter, URLParameters
from nozomi import Immutable, ApiRequest, RequestCredentials
from draft_sport.fantasy.player.profile import Profile
from typing import Type, TypeVar, Any, Optional, List
from draft_sport.ancillary.configuration import Configuration
from draft_sport.fantasy.scores.player.order_by import OrderBy
from draft_sport.fantasy.scores.player.points import Points
from draft_sport.data.order import Order

T = TypeVar('T', bound='Player')


class Player(Decodable):

    _PATH = '/fantasy/player'
    _LIST_PATH = '/fantasy/player/list'

    def __init__(
        self,
        profile: Profile,
        limit: int,
        offset: int,
        query_count: int,
        sequence: int,
        requesting_agent_id: str,
        points: Points,
        query_time: int
    ) -> None:

        self._profile = profile
        self._limit = limit
        self._offset = offset
        self._query_count = query_count
        self._sequence = sequence
        self._requesting_agent_id = requesting_agent_id
        self._points = points
        self._query_time = query_time

        return

    profile = Immutable(lambda s: s._profile)
    limit = Immutable(lambda s: s._limit)
    offset = Immutable(lambda s: s._offset)
    query_count = Immutable(lambda s: s._query_count)
    sequence = Immutable(lambda s: s._sequence)
    requesting_agent_id = Immutable(lambda s: s._requesting_agent_id)
    points = Immutable(lambda s: s._points)
    query_time = Immutable(lambda s: s._query_time)

    query_time_seconds = Immutable(
        lambda s: str(int(s._query_time) / 1000)
    )

    @classmethod
    def decode(cls: Type[T], data: Any) -> T:
        return cls(
            profile=Profile.decode(data['player']),
            limit=data['limit'],
            offset=data['offset'],
            query_count=data['query_count'],
            sequence=data['sequence'],
            requesting_agent_id=data['requesting_agent_id'],
            points=Points.decode(data['points']),
            query_time=data['query_time']
        )

    @classmethod
    def retrieve(
        cls: Type[T],
        public_id: str,
        season_id: str,
        credentials: Optional[RequestCredentials],
        configuration: Optional[Configuration]
    ) -> Optional[T]:

        parameters = URLParameters([
            URLParameter('public_id', public_id),
            URLParameter('season', season_id)
        ])

        request = ApiRequest(
            path=cls._PATH,
            method=HTTPMethod.GET,
            data=None,
            url_parameters=parameters,
            credentials=credentials,
            configuration=configuration
        )

        return cls.optionally_decode(request.response_data)

    @classmethod
    def retrieve_many(
        cls: Type[T],
        season_id: str,
        offset: int,
        limit: int,
        order_by: OrderBy = OrderBy.TOTAL_SEASON_POINTS,
        order: Order = Order.DESCENDING,
        name_fragment: Optional[str] = None,
        credentials: Optional[RequestCredentials] = None,
        configuration: Optional[Configuration] = None
    ) -> List[T]:

        parameters = URLParameters([
            URLParameter('season', season_id),
            URLParameter('offset', offset),
            URLParameter('limit', limit),
            URLParameter('order_by', order_by.value),
            URLParameter('order', order.value)
        ])

        if name_fragment is not None:
            parameters.append(URLParameter('fragment', name_fragment))

        request = ApiRequest(
            path=cls._LIST_PATH,
            method=HTTPMethod.GET,
            data=None,
            url_parameters=parameters,
            credentials=credentials,
            configuration=configuration
        )

        return cls.optionally_decode_many(
            request.response_data,
            default_to_empty_list=True
        )



"""
Draft Sport
Pick Module
author: hugh@blinkybeach.com
"""
from draft_sport.fantasy.scores.player.player import Player
from nozomi import Decodable, NozomiTime
from typing import TypeVar, Type, Any

T = TypeVar('T', bound='Pick')


class Pick(Decodable):

    def __init__(
        self,
        created: NozomiTime,
        manager_id: str,
        player: Player,
        league_id: str
    ) -> None:

        self._created = created
        self._manager_id = manager_id
        self._player = player
        self._league_id = league_id

        return

    @classmethod
    def decode(cls: Type[T], data: Any) -> T:
        return cls(
            created=NozomiTime.decode(data['created']),
            manager_id=data['manager_id'],
            player=Player.decode(data['player']),
            league_id=data['league_id']
        )



"""
Draft Sport Python
Player Round Points Module
author: hugh@blinkybeach.com
"""
from draft_sport.fantasy.scores.player.score import Score
from typing import List, TypeVar, Type, Any
from nozomi import Decodable, Immutable

T = TypeVar('T', bound='Round')


class Round(Decodable):
    """Points attributed to a player in a particular round"""

    def __init__(
        self,
        sequence: int,
        scores: List[Score]
    ) -> None:

        self._round_sequence = sequence
        self._scores = scores

        return

    round_sequence = Immutable(lambda s: s._round_sequence)
    scores = Immutable(lambda s: s._scores)

    @classmethod
    def decode(cls: Type[T], data: Any) -> T:
        return cls(
            sequence=data['round_sequence'],
            scores=Score.decode_many(data['scores'])
        )



"""
Draft Sport
Team Module
author: hugh@blinkybeach.com
"""
from typing import List, Any, TypeVar, Type
from nozomi import Immutable, Decodable
from draft_sport.leagues.pick import Pick

T = TypeVar('T', bound='Team')


class Team(Decodable):

    def __init__(
        self,
        league_id: str,
        picks: List[Pick],
        manager_id: str
    ) -> None:

        self._league_id = league_id
        self._picks = picks
        self._manager_id = manager_id

        return

    @classmethod
    def decode(cls: Type[T], data: Any) -> T:
        return cls(
            league_id=data['league_id'],
            picks=Pick.decode_many(data['picks']),
            manager_id=data['manager_id']
        )



"""
Draft Sport
Invitation Module
author: hugh@blinkybeach.com
"""
from nozomi import Immutable, Configuration, RequestCredentials
from nozomi import Decodable, NozomiTime
from typing import Optional, Type, TypeVar, Any, List
from nozomi import URLParameter, URLParameters, HTTPMethod, ApiRequest
from draft_sport.leagues.team import Team

T = TypeVar('T', bound='Inviation')


class Invitation(Decodable):

    _PATH = '/league/manager/invitation'

    def __init__(
        self,
        created: NozomiTime,
        league_name: str,
        league_id: str,
        token: str
    ) -> None:

        self._created = created
        self._league_name = league_name
        self._league_id = league_id
        self._token = token

        return

    created = Immutable(lambda s: s._created)
    league_name = Immutable(lambda s: s._league_name)
    league_id = Immutable(lambda s: s._league_id)
    token = Immutable(lambda s: s._token)

    @classmethod
    def retrieve(
        cls: Type[T],
        token: str,
        credentials: RequestCredentials,
        configuration: Configuration
    ) -> Optional[T]:
        """
        Optionally return an Invitation with the given token if it exists
        """

        assert isinstance(token, str)

        parameters = URLParameters([URLParameter('token', token)])

        request = ApiRequest(
            path=cls._PATH,
            method=HTTPMethod.GET,
            configuration=configuration,
            data=None,
            url_parameters=parameters,
            credentials=credentials
        )

        return cls.optionally_decode(request.response_data)

    @classmethod
    def decode(cls: Type[T], data: Any) -> T:
        return cls(
            created=NozomiTime.decode(data['created']),
            token=data['token'],
            league_id=data['league_public_id'],
            league_name=data['league_name']
        )
